n = 4 # number of circles

def constants(n):
    outstr = "\tConstants\n"
    outstr += "\t\tn = {0}\t ! number of circles \n".format(n) 
    outstr += "\tEnd Constants\n"
    return outstr

variables_str = """
    Variables
        x[1:n] >= -2.0, <= 2.0      ! x-coordinates of circle centers
        y[1:n] >= -2.0, <= 2.0      ! y-coordinates of circle centers
        circrad = 2.0, >= 1.7, <= 2.5   ! radius of circumscribing circle
    End Variables
"""
        
intermediates_str = """
    Intermediates
        ints[1] = 1
        ints[2:n] = ints[1:n-1] + 1
        r[1:n] = ints[1:n]**(-0.5)  ! radii of circles
    End Intermediates
"""
    
def cons1(n):
    outstr = "\t\t ! distance between pairs of circles is >= sum of their radii\n"
    for i in range(2,n+1):
        for j in range(1,i):
            outstr += "\t\tsqrt((x[{0}] - x[{1}])^2 + (y[{0}] - y[{1}])^2) >= r[{0}] + r[{1}]\n".format(i, j)
    return outstr

cons2_str = """
        ! circles are inside circumscribing circle
        circrad - sqrt(x[1:n]^2 + y[1:n]^2) >= r[1:n]
        
"""

def gencircleapm(n):
    outstr = "Model Pack Circles\n"
    outstr += constants(n)
    outstr += variables_str
    outstr += intermediates_str
    outstr += "\n\tEquations\n";
    outstr += cons1(n)
    outstr += cons2_str
    outstr += "\t\tminimize circrad\n"
    outstr += "\tEnd Equations\nEndModel"
    return outstr

modelstr = gencircleapm(n)  # generate apm file
f = open('packcircles.apm', mode = 'w')
f.write(modelstr)
f.close()


# Import APM Python library apm.py
from apm import *
# Select the server
server = 'http://xps.apmonitor.com'
# Give the application a name
app = 'packcircles'
# Clear any previous applications by that name
apm(server,app,'clear all')
# Load the model file
apm_load(server,app,'packcircles.apm')
apm_option(server,app,'nlc.solver',1)
apm_option(server,app,'nlc.coldstart',1)
apm_option(server,app,'nlc.max_iter',500)
# Solve on APM server
solver_output = apm(server,app,'solve')
# Display solver output
print solver_output
# Retrieve results
(results, array) = apm_sol(server,app)

# get results from python array

circrad = array['packcircles.circrad'][0]   # radius of circumscribing circles
xs = [] # x coordinates of circle centers
ys = [] # y coordinates of circle centers
for i in range(1,n+1):
    strx = 'packcircles.x[{0}]'.format(i)
    stry = 'packcircles.y[{0}]'.format(i)
    xs.append(array[strx][0])
    ys.append(array[stry][0])

# plot results

import pylab
pylab.axes()
cc = pylab.Circle((0,0),circrad,facecolor = 'none')
pylab.gca().add_patch(cc)
for i in range(n):
     cc = pylab.Circle((xs[i],ys[i]),(i+1)**(-0.5),facecolor = 'none')
     pylab.gca().add_patch(cc)
     
pylab.axis('scaled')
pylab.show()
